//****************************************************************************
//**                                                                         *
//**  FILE        :  tc33x.lsl                                               *
//**                                                                         *
//**  DESCRIPTION :  LSL description: Infineon TC33X                         *
//**                 This 2nd Generation AURIX derivative has:               *
//**                            1 TriCore cores                              *
//**                            1 XC800 cores                                *
//**                                                                         *
//**  Copyright 2011-2021 TASKING BV                                         *
//**                                                                         *
//**  Configuration macros:                                                  *
//**    __NO_VTC                                                             *
//**      When this macro is defined, the virtual core vtc is not available, *
//**      so a separate link task is needed for each TriCore core            *
//**      (default: undefined).                                              *
//**    USTACK_TC0                                                           *
//**      Size of the global stack for core 0                                *
//**      (default: 16k for core 0, 1k for other cores).                     *
//**    ISTACK_TC0                                                           *
//**      Interrupt stack size for core 0 (default: 1k).                     *
//**    HEAP                                                                 *
//**      Size of the heap, available when macro __NO_VTC is not defined     *
//**      (default: 16k)                                                     *
//**    HEAP_TC0                                                             *
//**      Size of the heap for core 0, available when macro __NO_VTC is      *
//**      defined (default: 16k)                                             *
//**    A0_START                                                             *
//**      Fixed address of the A0-addressable segment for all cores          *
//**      (default: not set).                                                *
//**    A1_START                                                             *
//**      Fixed address of the A1-addressable segment for all cores          *
//**      (default: not set).                                                *
//**    A8_START                                                             *
//**      Fixed address of the A8-addressable segment for all cores          *
//**      (default: not set).                                                *
//**    A9_START                                                             *
//**      Fixed address of the A9-addressable segment for all cores          *
//**      (default: not set).                                                *
//**    __TC0_A0_START                                                       *
//**      Fixed address of the A0-addressable segment for core 0,            *
//**      available when macro __NO_VTC is defined (default: not set).       *
//**    __TC0_A1_START                                                       *
//**      Fixed address of the A1-addressable segment for core 0,            *
//**      available when macro __NO_VTC is defined (default: not set).       *
//**    __TC0_A8_START                                                       *
//**      Fixed address of the A8-addressable segment for core 0,            *
//**      available when macro __NO_VTC is defined (default: not set).       *
//**    __TC0_A9_START                                                       *
//**      Fixed address of the A9-addressable segment for core 0,            *
//**      available when macro __NO_VTC is defined (default: not set).       *
//**                                                                         *
//****************************************************************************

#ifdef __NO_VTC
#define __CORE0 tc0
#else
#define __CORE0 vtc
#endif

#include "tc1v1_6_2.lsl"

#ifndef __VSTACK_XDATA
#define __VSTACK_XDATA 1k
#endif

#include "arch_scr2g.lsl"

#ifndef XC800INIT_FILL
#define XC800INIT_FILL          0
#endif

#define SCR_BASE_ADDR           0xf0240000

#ifndef    __REDEFINE_ON_CHIP_ITEMS

// Specify a multi-core processor environment (mpe)

processor mpe
{
        derivative = tc33x;
}
#endif  // __REDEFINE_ON_CHIP_ITEMS

#ifndef CSA_TC0
#define CSA_TC0         64                      /* number of context blocks tc0 */
#endif

#ifndef CSA_START_TC0
#define CSA_START_TC0   0xd0004000              /* start address of CSA tc0 */
#endif

#ifndef USTACK_TC0
#define USTACK_TC0      16k                     /* user stack size tc0 */
#endif


#ifndef ISTACK_TC0
#define ISTACK_TC0      1k                      /* interrupt stack size tc0 */
#endif

#ifndef HEAP
#define HEAP            16k                     /* heap size */
#endif

#ifndef HEAP_TC0
#define HEAP_TC0        16k                     /* heap size tc0 */
#endif


/*
 * Define __MAX_CONCURRENT_HANDLERS to the number of interrupt and trap handlers
 * that should contribute to stack size estimation. The handlers that use the
 * most stack space are selected. Allowed values: 0 or higher. Default: not set.
 * Defining this macro adds a threads attribute to ustack, with the macro value
 * increased by one.
 */
#ifndef __MAX_CONCURRENT_HANDLERS
#define __USTACK0_THREADS_ATTRIBUTE
#else
#define __USTACK0_THREADS_ATTRIBUTE ,threads=(__MAX_CONCURRENT_HANDLERS)+1
#endif

/*
 * If a separate program is run on a specific core <n>, then the stack usage of this program
 * can be computed separately by defining LSL macro __USTACK<n>_ENTRY_POINTS to the name of the
 * symbol (between double quotes) that represents the main function for this program. Multiple
 * symbols can be specified by listing them between brackets, separated by commas.
 */
#ifdef __USTACK0_ENTRY_POINTS
#define __USTACK0_ENTRY_POINTS_ATTRIBUTE ,entry_points=__USTACK0_ENTRY_POINTS
#else
#ifdef __NO_VTC
#define __USTACK0_ENTRY_POINTS_ATTRIBUTE ,entry_points=["_START" /* , group trap_tab, group int_tab */ ]
#else
#define __USTACK0_ENTRY_POINTS_ATTRIBUTE ,entry_points=["_START" /* , group trap_tab_tc0, group int_tab_tc0 */ ]
#endif
#endif
#ifdef __ISTACK0_ENTRY_POINTS
#define __ISTACK0_ENTRY_POINTS_ATTRIBUTE ,entry_points=__ISTACK0_ENTRY_POINTS
#else
#define __ISTACK0_ENTRY_POINTS_ATTRIBUTE
#endif

#ifndef XVWBUF
#define XVWBUF          0                       /* buffer used by debugger */
#endif

#ifndef INTTAB
#define INTTAB          0xa00f0000              /* start address of interrupt table */
#endif
#ifndef TRAPTAB
#define TRAPTAB         (INTTAB + 0x2000)      /* start address of trap table */
#endif

#ifndef INTTAB0
#define INTTAB0         (INTTAB)
#endif

#ifndef TRAPTAB0
#define TRAPTAB0        (TRAPTAB)
#endif

#ifndef RESET
#define RESET           0x80040000              /* internal flash start address tc0 */
#endif

// Eclipse generates Ax_START_ADDRESS,
// users may define Ax_START on the command line
#ifndef A0_START_ADDRESS
#ifdef  A0_START
#define A0_START_ADDRESS A0_START
#endif
#endif
#ifndef A1_START_ADDRESS
#ifdef  A1_START
#define A1_START_ADDRESS A1_START
#endif
#endif
#ifndef A8_START_ADDRESS
#ifdef  A8_START
#define A8_START_ADDRESS A8_START
#endif
#endif
#ifndef A9_START_ADDRESS
#ifdef  A9_START
#define A9_START_ADDRESS A9_START
#endif
#endif

#ifndef __NO_VTC
#ifdef A0_START_ADDRESS
#define __A0_START_ADDRESS A0_START_ADDRESS
#endif
#ifdef A1_START_ADDRESS
#define __A1_START_ADDRESS A1_START_ADDRESS
#endif
#ifdef A8_START_ADDRESS
#define __A8_START_ADDRESS A8_START_ADDRESS
#endif
#ifdef A9_START_ADDRESS
#define __A9_START_ADDRESS A9_START_ADDRESS
#endif
#endif


derivative tc33x
{
        core tc0
        {
                architecture = TC1V1.6.2;
#ifndef __NO_VTC
                space_id_offset = 100;  // add 100 to all space IDs in the architecture definition
                copytable_space = vtc:linear;   // use the copy table in the virtual core for 'bss' and initialized data sections
#endif
        }


#ifndef __NO_VTC
        core vtc
        {
                architecture = TC1V1.6.2;
                import tc0;                      // add all address spaces of core tc0 to core vtc for linking and locating
        }
#endif

        core xc800
        {
                architecture = scr2g;
        }

        bus sri
        {
                mau = 8;
                width = 32;

                // segments starting from 0x0
                map (dest=bus:tc0:fpi_bus, src_offset=0, dest_offset=0, size=0x70000000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0, dest_offset=0, size=0x80000000);
#endif

                // local memory mapped to individual cores
                map (dest=bus:tc0:fpi_bus, src_offset=0x70000000, dest_offset=0x70000000, size=0x00100000, priority=3, exec_priority=0);
                map (dest=bus:tc0:fpi_bus, src_offset=0x70100000, dest_offset=0x70100000, size=0x0ff00000, exec_priority=3);
                map (dest=bus:tc0:fpi_bus, src_offset=0x30000000, dest_offset=0x30000000, size=0x40000000);
                map (dest=bus:tc0:fpi_bus, src_offset=0x10000000, dest_offset=0x10000000, size=0x10000000);

                // pflash
                map (dest=bus:tc0:fpi_bus, src_offset=0x80000000, dest_offset=0x80000000, size=0x300000, priority=2);
                map (dest=bus:tc0:fpi_bus, src_offset=0x80300000, dest_offset=0x80300000, size=0xfd00000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0x80000000, dest_offset=0x80000000, size=0x10000000);
#endif // __NO_VTC

                // dlmuram
                map (dest=bus:tc0:fpi_bus, src_offset=0x90000000, dest_offset=0x90000000, size=0x10000, priority=2);
                map (dest=bus:tc0:fpi_bus, src_offset=0x90010000, dest_offset=0x90010000, size=0x30000);
                map (dest=bus:tc0:fpi_bus, src_offset=0x90100000, dest_offset=0x90100000, size=0x20000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0x90000000, dest_offset=0x90000000, size=0x40000, priority=1);
                map (dest=bus:vtc:fpi_bus, src_offset=0x90100000, dest_offset=0x90100000, size=0x20000, priority=1);
#endif // __NO_VTC

                // lmuram
                map (dest=bus:tc0:fpi_bus, src_offset=0x90040000, dest_offset=0x90040000, size=0xc0000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0x90040000, dest_offset=0x90040000, size=0xc0000, priority=1);
#endif

                // remaining segment 9 areas
                map (dest=bus:tc0:fpi_bus, src_offset=0x90120000, dest_offset=0x90120000, size=0xfee0000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0x90120000, dest_offset=0x90120000, size=0xfee0000);
#endif // __NO_VTC

                // pflash and dlmuram
                map (dest=bus:tc0:fpi_bus, src_offset=0xa0000000, dest_offset=0xa0000000, size=0x20000000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0xa0000000, dest_offset=0xa0000000, size=0x20000000);
#endif // __NO_VTC

                // segments starting from 0xe
                map (dest=bus:tc0:fpi_bus, src_offset=0xe0000000, dest_offset=0xe0000000, size=0x20000000);
#ifndef __NO_VTC
                map (dest=bus:vtc:fpi_bus, src_offset=0xe0000000, dest_offset=0xe0000000, size=0x20000000);
#endif // __NO_VTC
        }

#ifndef    __REDEFINE_ON_CHIP_ITEMS
#ifndef __CPP_RUN_TIME_ENTRY_FLAG 
#define __CPP_RUN_TIME_ENTRY_FLAG mem:dspr0
#endif 

#ifndef __NO_VTC
        section_layout :vtc:linear
        {
                group (ordered, run_addr=__CPP_RUN_TIME_ENTRY_FLAG)
                {
                        // C++ run-time variable "main_called" that ensures the global object constructors to execute exactly once.
                        // main_called is initialized so its name gets a data prefix: .sect '.data.__section_main_called'
                        select ".data.__section_main_called";
                        // C++ run-time variables to make destructors concurrent
                        select ".data.__section_dtor_finalizer";
                }
        }
#else
        section_layout :tc0:linear
        {
                group (ordered, run_addr=__CPP_RUN_TIME_ENTRY_FLAG)
                {
                        // C++ run-time variable "main_called" that ensures the global object constructors to execute exactly once.
                        // main_called is initialized so its name gets a data prefix: .sect '.data.__section_main_called'
                        select ".data.__section_main_called";
                        // C++ run-time variables to make destructors concurrent
                        select ".data.__section_dtor_finalizer";
                }
        }
#endif  // __NO_VTC
#endif  // __REDEFINE_ON_CHIP_ITEMS

#ifndef    __REDEFINE_ON_CHIP_ITEMS

        memory dspr0 // Data Scratch Pad Ram CPU0
        {
                mau = 8;
                size = 192k;
                type = ram;
                priority = 1;
                exec_priority = 0;
                map (dest=bus:tc0:fpi_bus, dest_offset=0xd0000000, size=192k, priority=1, exec_priority=0);
                map (dest=bus:sri, dest_offset=0x70000000, size=100k);
        }

        memory pspr0 // Program Scratch Pad Ram CPU0
        {
                mau = 8;
                size = 8k;
                type = ram;
                priority = 0;
                exec_priority = 1;
                map (dest=bus:tc0:fpi_bus, dest_offset=0xc0000000, size=8k, exec_priority=1);
                map (dest=bus:sri, dest_offset=0x70100000, size=8k);
        }

        memory pflash0 // Program Flash CPU0
        {
                mau = 8;
                size = 2M;
                type = rom;
				map (dest=bus:sri, dest_offset=PFLASH0_START,           size=PFLASH0_SIZE);
                //map     cached (dest=bus:sri, dest_offset=0x80000000,           size=2M, cached);
                //map not_cached (dest=bus:sri, dest_offset=0xa0000000, reserved, size=2M);
        }

        memory brom
        {
                mau = 8;
                size = 64k;
                type = reserved rom;
                map     cached (dest=bus:sri, dest_offset=0x8fff0000,           size=64k, cached);
                map not_cached (dest=bus:sri, dest_offset=0xafff0000, reserved, size=64k);
        }

        memory dflash0
        {
                mau = 8;
                size = 128k;
                type = reserved nvram;
                map (dest=bus:sri, dest_offset=0xaf000000, size=128k);
        }

        memory dflash0_ucb
        {
                mau = 8;
                size = 24k;
                type = reserved nvram;
                map (dest=bus:sri, dest_offset=0xaf400000, size=24k);
        }

        memory dflash0_cfs
        {
                mau = 8;
                size = 64k;
                type = reserved nvram;
                map (dest=bus:sri, dest_offset=0xaf800000, size=64k);
        }

        memory dflash1
        {
                mau = 8;
                size = 128k;
                type = reserved nvram;
                map (dest=bus:sri, dest_offset=0xafc00000, size=128k);
        }

        memory dlmucpu0 // DLMU CPU0
        {
                mau = 8;
                size = 8k;
                type = ram;
                map     cached (dest=bus:sri, dest_offset=0x90000000,           size=8k, cached);
                map not_cached (dest=bus:sri, dest_offset=0xb0000000, reserved, size=8k);
        }

        memory scr_iram
        {
                mau = 8;
                type = ram;
                size = 256;
                map ( dest=bus:xc800:idata_bus, src_offset=0x0, dest_offset=0x0, size=256 );
        }

        memory scr_xram
        {
                mau = 8;
                type = nvram;
                size = 8k;
                map ( dest=bus:xc800:xdata_bus, src_offset=0x0, dest_offset=0x0, size=8k );
                map ( dest=bus:xc800:code_bus, src_offset=0x0, dest_offset=0x0, size=8k );
                map ( dest=bus:sri, dest_offset=SCR_BASE_ADDR, src_offset=0, size=8k, reserved );
        }

        memory scr_bootrom
        {
                mau = 8;
                type = reserved rom;
                size = 2k;
                map ( dest=bus:xc800:code_bus, src_offset=0x0, dest_offset=0xd000, size=2k );
        }

#endif  // __REDEFINE_ON_CHIP_ITEMS
#ifndef __NO_VTC
        section_setup :vtc:linear
        {
#ifndef __LINKONLY__
                start_address
                (
                        run_addr = (RESET),
                        symbol = "_START"
                );
#endif
        }
#else
        section_setup :tc0:linear
        {
#ifndef __LINKONLY__
                start_address
                (
                        run_addr = (RESET),
                        symbol = "_START"
                );
#endif
        }
#endif

#ifndef __NO_VTC
        section_setup :tc0:linear
        {
                stack "ustack_tc0"
                (
                        min_size = (USTACK_TC0)
                        ,fixed
                        ,align = 8
                        __USTACK0_THREADS_ATTRIBUTE
                        __USTACK0_ENTRY_POINTS_ATTRIBUTE
                );

                stack "istack_tc0"
                (
                        min_size = (ISTACK_TC0)
                        ,fixed
                        ,align = 8
                        __ISTACK0_ENTRY_POINTS_ATTRIBUTE
                );
        }

        section_setup :vtc:linear
        {
                heap "heap"
                (
                        min_size = (HEAP),
                        fixed,
                        align = 8
                );
        }
#else
        section_setup :tc0:linear
        {
                stack "ustack"
                (
                        min_size = (USTACK_TC0)
                        , fixed
                        , align = 8
                        __USTACK0_THREADS_ATTRIBUTE
                        __USTACK0_ENTRY_POINTS_ATTRIBUTE
                );

                stack "istack"
                (
                        min_size = (ISTACK_TC0)
                        ,fixed
                        ,align = 8
                        __ISTACK0_ENTRY_POINTS_ATTRIBUTE
                );
                heap "heap"
                (
                        min_size = (HEAP_TC0),
                        fixed,
                        align = 8
                );
        }

#endif
#ifndef __NO_VTC
        section_layout :tc0:linear
        {
                "_lc_ub_ustack" := "_lc_ub_ustack_tc0"; /* common cstart interface for first or single core */
                "_lc_ue_ustack" := "_lc_ue_ustack_tc0"; /* common cstart interface for first or single core */
                "_lc_ue_istack" := "_lc_ue_istack_tc0"; /* common cstart interface for first or single core */
        }
#endif

#ifndef __NO_VTC
        section_setup :vtc:linear
        {
                copytable
                (
                        align = 4,
                        dest = linear
                );

                mpu_data_table;
        }
        section_layout :tc0:csa
        {
                group  (ordered, align = 64, attributes=rw, run_addr=(CSA_START_TC0))
                        reserved "csa_tc0" (size = 64 * (CSA_TC0));
                "_lc_ub_csa_01" := "_lc_ub_csa_tc0"; /* common cstart interface for first or single core */
                "_lc_ue_csa_01" := "_lc_ue_csa_tc0"; /* common cstart interface for first or single core */
        }
#else
        section_setup :tc0:linear
        {
                copytable
                (
                        align = 4,
                        dest = linear
                );

                mpu_data_table;
        }

        section_layout :tc0:csa
        {
                group  (ordered, align = 64, attributes=rw, run_addr=(CSA_START_TC0))
                        reserved "csa_tc0" (size = 64 * (CSA_TC0));
                "_lc_ub_csa_01" := "_lc_ub_csa_tc0"; /* common cstart interface for first or single core */
                "_lc_ue_csa_01" := "_lc_ue_csa_tc0"; /* common cstart interface for first or single core */
        }
#endif

#ifndef __NO_VTC
        section_layout :vtc:linear
        {
#if (XVWBUF>0)
                group (align = 4) reserved "_xvwbuffer_" (size=XVWBUF, attributes=rw);
                "_lc_ub_xvwbuffer" = "_lc_ub__xvwbuffer_";
                "_lc_ue_xvwbuffer" = "_lc_ue__xvwbuffer_";
#endif

#ifndef __REDEFINE_BASE_ADDRESS_GROUPS
#include        "base_address_groups.lsl"
#endif

                "_SMALL_DATA_TC0" := "_SMALL_DATA_";

                "_LITERAL_DATA_TC0" := "_LITERAL_DATA_";

                "_A8_DATA_TC0" := "_A8_DATA_";

                "_A9_DATA_TC0" := "_A9_DATA_";
        }
#else // __NO_VTC
        section_layout :tc0:linear
        {
#if (XVWBUF>0)
                group (align = 4) reserved "_xvwbuffer_" (size=XVWBUF, attributes=rw);
                "_lc_ub_xvwbuffer" = "_lc_ub__xvwbuffer_";
                "_lc_ue_xvwbuffer" = "_lc_ue__xvwbuffer_";
#endif

#ifndef __REDEFINE_BASE_ADDRESS_GROUPS
#undef __A0_START_ADDRESS
#ifdef __TC0_A0_START
#define __A0_START_ADDRESS __TC0_A0_START
#else
#ifdef A0_START_ADDRESS
#define __A0_START_ADDRESS A0_START_ADDRESS
#endif
#endif
#undef __A1_START_ADDRESS
#ifdef __TC0_A1_START
#define __A1_START_ADDRESS __TC0_A1_START
#else
#ifdef A1_START_ADDRESS
#define __A1_START_ADDRESS A1_START_ADDRESS
#endif
#endif
#undef __A8_START_ADDRESS
#ifdef __TC0_A8_START
#define __A8_START_ADDRESS __TC0_A8_START
#else
#ifdef A8_START_ADDRESS
#define __A8_START_ADDRESS A8_START_ADDRESS
#endif
#endif
#undef __A9_START_ADDRESS
#ifdef __TC0_A9_START
#define __A9_START_ADDRESS __TC0_A9_START
#else
#ifdef A9_START_ADDRESS
#define __A9_START_ADDRESS A9_START_ADDRESS
#endif
#endif
#include        "base_address_groups.lsl"
#endif
        }
#endif // __NO_VTC

#ifndef __NO_VTC
        section_layout :vtc:linear
        {
                        "_lc_u_int_tab_tc0" = (INTTAB0);
                        "_lc_u_int_tab" = "_lc_u_int_tab_tc0"; /* common cstart interface for first or single core */

                        // interrupt vector tables
                        group int_tab_tc0 (ordered)
                        {
#                               include "inttab0.lsl"
                        }

                        "_lc_u_trap_tab_tc0" = (TRAPTAB0);
                        "_lc_u_trap_tab" = "_lc_u_trap_tab_tc0"; /* common cstart interface for first or single core */

                        // trapvector tables
                        group trap_tab_tc0 (ordered)
                        {
#                               //include "traptab0.lsl"
                        }
        }
#else
        section_layout :tc0:linear
        {
                        "_lc_u_int_tab" = (INTTAB0);
                        "_lc_u_int_tab_tc0" = (INTTAB0);
                        // interrupt vector tables
                        group int_tab (ordered)
                        {
#                               include "inttab0.lsl"
                        }
                        "_lc_u_trap_tab" = (TRAPTAB0);
                        "_lc_u_trap_tab_tc0" = (TRAPTAB0);

                        // trapvector tables
                        group trap_tab (ordered)
                        {
#                               //include "traptab0.lsl"
                        }
        }
#endif // __NO_VTC

#ifndef NOXC800INIT
        section_layout :__CORE0:linear
        {
                group (ordered, align=4) memcopy ".rodata.xc800init" (memory = scr_xram, fill=XC800INIT_FILL);
        }
#endif
        section_layout :__CORE0:linear
        {
#include        "tc1v1_6_2.bmhd.lsl"
        }

#if !defined(__DISABLE_SCR_BOOT_MAGIC)
        /*
         *      The last 8 bytes of SCR XRAM starting at address 0x01FF8 must contain
         *      4 pairs of bytes where each pair consists of 0x55 followed by 0xAA.
         *      The user code will not be executed and the SCR will enter an endless
         *      loop if the memory content does not match this data sequence. This
         *      feature is meant to avoid an unintentional entry into User Mode 1.
         *      When the 8 bytes match, the SCR boot code will trigger an interrupt
         *      to the TriCore by setting bit NMICON.SCRINTTC to 1 with a value of 0x80
         *      in the SCRINTEXCHG register. When the 8 bytes do not match, the same
         *      interrupt is triggered with a value of 0x81 in the SCRINTEXCHG register.
         */
        section_layout :xc800:xdata
        {
                group(ordered, run_addr=0x1ff8)
                {
                        struct "scr_boot_magic"
                        {
                                0x55:1; 0xaa:1;
                                0x55:1; 0xaa:1;
                                0x55:1; 0xaa:1;
                                0x55:1; 0xaa:1;
                        }
                }
        }
#endif
}

//
//      The following macros are required for extmem.lsl
//
#define HAS_ON_CHIP_FLASH       // This derivative has on-chip flash
#define HAS_NO_EXTERNAL_RAM     // Exclude xram_8_a

